import { AuthAdapter, CryptoAdapter, DbAdapter, FetchPolyfill, StorageAdapter, CoreLibrary } from './adapters';
import { findEntityById } from './findEntityById';
import { fetchEntityById } from './fetchEntityById';
import { queryById } from './queryById';
import { expandEntity } from './expandEntity';
import { getPrivateKey } from './getPrivateKey';
import { getCollectionItems } from './getCollectionItems';
import { getPaginatedCollectionItems } from './getPaginatedCollectionItems';
import { expandCollection } from './expandCollection';
import { getActorByUserId } from './getActorByUserId';
import { broadcast } from './broadcast';
import { getRecipientUrls } from './getRecipientUrls';
import { getRecipientInboxUrls } from './getRecipientInboxUrls';
import { signAndSendToForeignActorInbox } from './signAndSendToForeignActorInbox';
export declare class Core implements CoreLibrary {
    fetch: FetchPolyfill;
    initializeDb?: DbAdapter['initializeDb'];
    findAll: DbAdapter['findAll'];
    findOne: DbAdapter['findOne'];
    findStringIdByValue: DbAdapter['findStringIdByValue'];
    findStringValueById: DbAdapter['findStringValueById'];
    insertItem: DbAdapter['insertItem'];
    removeItem: DbAdapter['removeItem'];
    insertOrderedItem: DbAdapter['insertOrderedItem'];
    removeOrderedItem: DbAdapter['removeOrderedItem'];
    saveEntity: DbAdapter['saveEntity'];
    saveString: DbAdapter['saveString'];
    getTokenByUserId: AuthAdapter['getTokenByUserId'];
    createUser: AuthAdapter['createUser'];
    getUserIdByToken: AuthAdapter['getUserIdByToken'];
    authenticatePassword: AuthAdapter['authenticatePassword'];
    generateKeyPair: CryptoAdapter['generateKeyPair'];
    getHttpSignature: CryptoAdapter['getHttpSignature'];
    hashPassword: CryptoAdapter['hashPassword'];
    randomBytes: CryptoAdapter['randomBytes'];
    upload: StorageAdapter['upload'];
    getGuid: () => Promise<string>;
    constructor(adapters: {
        auth: AuthAdapter;
        storage: StorageAdapter;
        crypto: CryptoAdapter;
        db: DbAdapter;
        fetch?: FetchPolyfill;
    });
    findEntityById: typeof findEntityById;
    getActorByUserId: typeof getActorByUserId;
    getPrivateKey: typeof getPrivateKey;
    getStreamByName: (this: Core, actor: import("@activity-kit/types").Actor, name: string) => Promise<import("@activity-kit/types").EitherCollection>;
    fetchEntityById: typeof fetchEntityById;
    queryById: typeof queryById;
    expandEntity: typeof expandEntity;
    getCollectionItems: typeof getCollectionItems;
    getPaginatedCollectionItems: typeof getPaginatedCollectionItems;
    expandCollection: typeof expandCollection;
    getRecipientInboxUrls: typeof getRecipientInboxUrls;
    getRecipientUrls: typeof getRecipientUrls;
    broadcast: typeof broadcast;
    signAndSendToForeignActorInbox: typeof signAndSendToForeignActorInbox;
}
export { AuthAdapter, CryptoAdapter, DbAdapter, DbOptions, FetchPolyfill, StorageAdapter, CoreLibrary, Plugin, Routes, } from './adapters';
